package vue;

// Native-backed emulation core implementation
class NativeVue extends Vue {

    // Instance fields
    private long handle; // Context address in native memory



    ///////////////////////////////////////////////////////////////////////////
    //                             Constructors                              //
    ///////////////////////////////////////////////////////////////////////////

    // Default constructor
    private native long construct();
    NativeVue() {
        handle = construct();
    }



    ///////////////////////////////////////////////////////////////////////////
    //                            Public Methods                             //
    ///////////////////////////////////////////////////////////////////////////

    // Release any used resources
    private native void dispose(long handle);
    public void dispose()
        { dispose(handle); }

    // Process the simulation
    private native int emulate(long handle, int maxCycles);
    public int emulate(int maxCycles)
        { return emulate(handle, maxCycles); }

    // Evaluate the condition in a breakpoint
    private native boolean evaluate(long handle, Breakpoint brk);
    public boolean evaluate(Breakpoint brk)
        { return evaluate(handle, brk); }

    // Retrieve the application break code
    private native int getBreakCode(long handle);
    public int getBreakCode() { return getBreakCode(handle); }

    // Retrieve a register value
    private native int getRegister(long handle, int index, boolean system);
    public int getRegister(int index, boolean system)
        { return getRegister(handle, index, system); }

    // Retrieve a copy of the ROM data
    private native byte[] getROM(long handle);
    public byte[] getROM() { return getROM(handle); }

    // Determine whether the context is native-backed
    public native boolean isNative();

    // Read a value from the CPU bus
    private native int read(long handle, int address, int type);
    public int read(int address, int type)
        { return read(handle, address, type); }

    // Read bytes from the CPU bus
    private native boolean readBytes(long handle, int address, byte[] dest,
        int offset, int length);
    public boolean readBytes(int address, byte[] dest, int offset, int length)
        { return readBytes(handle, address, dest, offset, length); }

    // Initialize all system components
    private native void reset(long handle);
    public void reset()
        { reset(handle); }

    // Specify a register value
    private native int setRegister(long handle, int index, boolean system,
        int value);
    public int setRegister(int index, boolean system, int value)
        { return setRegister(handle, index, system, value); }

    // Provide new ROM data
    private native boolean setROM(long handle, byte[] data, int offset,
        int length);
    public boolean setROM(byte[] data, int offset, int length)
        { return setROM(handle, data, offset, length); }

    // Write a value to the CPU bus
    private native void write(long handle, int address, int type, int value);
    public void write(int address, int type, int value)
        { write(handle, address, type, value); }

    // Write bytes to the CPU bus
    private native boolean writeBytes(long handle, int address, byte[] src,
        int offset, int length);
    public boolean writeBytes(int address, byte[] src, int offset, int length)
        { return writeBytes(handle, address, src, offset, length); }

}
